MODULE iscplrst
   !!======================================================================
   !!                       ***  MODULE  iscplrst***
   !! Ocean forcing: update the restart file in case of ice sheet/ocean coupling
   !!=====================================================================
   !! History :  NEMO  ! 2015-01 P. Mathiot: original 
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   iscpl_rst_interpol : restart interpolation in case of coupling with ice sheet
   !!----------------------------------------------------------------------
   USE dom_oce         ! ocean space and time domain
   USE domwri          ! ocean space and time domain
   USE domvvl, ONLY : dom_vvl_interpol
   USE phycst          ! physical constants
   USE sbc_oce         ! surface boundary condition variables
   USE oce             ! global tra/dyn variable
   USE in_out_manager  ! I/O manager
   USE iom             ! I/O module
   USE lib_mpp         ! MPP library
   USE lib_fortran     ! MPP library
   USE wrk_nemo        ! Memory allocation
   USE lbclnk
   USE domngb
   USE sbc_ice, ONLY : lk_lim3
   USE iscplini
   USE iscplhsb

   IMPLICIT NONE
   PRIVATE
   
   PUBLIC   iscpl_stp       
   PUBLIC   iscpl_rst_interpol ! routine to wet and dry
   !!
   !! * Substitutions  
#  include "domzgr_substitute.h90"  
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.3 , NEMO Consortium (2010)
   !! $Id: sbcrnf.F90 4666 2014-06-11 12:52:23Z mathiot $
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE iscpl_stp
      !!---------------------------------------------------------------------- 
      !!                   ***  ROUTINE iscpl_stp  ***
      !! 
      !! ** Purpose : compute initialisation
      !!              compute extrapolation of restart variable un, vn, tsn, sshn (wetting/drying)   
      !!              compute correction term if needed
      !! 
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(:,:  ), POINTER :: zsmask_b
      REAL(wp), DIMENSION(:,:,:), POINTER :: ztmask_b, zumask_b, zvmask_b
      REAL(wp), DIMENSION(:,:,:), POINTER :: ze3t_b  , ze3u_b  , ze3v_b  
      REAL(wp), DIMENSION(:,:,:), POINTER :: zdepw_b
!!----------------------------------------------------------------------
      INTEGER  ::   inum0
      CHARACTER(20) :: cfile

      CALL wrk_alloc(jpi,jpj,jpk, ztmask_b, zumask_b, zvmask_b) ! need this new variables for evoving isf cavity
      CALL wrk_alloc(jpi,jpj,jpk, ze3t_b  , ze3u_b  , ze3v_b  ) ! need of this variables for interpolation)
      CALL wrk_alloc(jpi,jpj,jpk, zdepw_b )
      CALL wrk_alloc(jpi,jpj,     zsmask_b                    )


      !! get restart variable
      CALL iom_get( numror, jpdom_autoglo, 'tmask'  , ztmask_b   ) ! need to extrapolate T/S
      CALL iom_get( numror, jpdom_autoglo, 'umask'  , zumask_b   ) ! need to correct barotropic velocity
      CALL iom_get( numror, jpdom_autoglo, 'vmask'  , zvmask_b   ) ! need to correct barotropic velocity
      CALL iom_get( numror, jpdom_autoglo, 'smask'  , zsmask_b   ) ! need to correct barotropic velocity
      CALL iom_get( numror, jpdom_autoglo, 'fse3t_n', ze3t_b(:,:,:) ) ! need to compute temperature correction
      CALL iom_get( numror, jpdom_autoglo, 'fse3u_n', ze3u_b(:,:,:) ) ! need to compute volume      correction  ????
      CALL iom_get( numror, jpdom_autoglo, 'fse3v_n', ze3v_b(:,:,:) ) ! need to compute volume      correction  ????
      CALL iom_get( numror, jpdom_autoglo, 'fsdepw_n', zdepw_b(:,:,:) ) ! need to compute volume      correction  ????

      !! read namelist
      CALL iscpl_init()

      !!  ! Extrapolation/interpolation of modify cell and new cells ... (maybe do it later after domvvl)
      CALL iscpl_rst_interpol( ztmask_b, zumask_b, zvmask_b, zsmask_b, ze3t_b, ze3u_b, ze3v_b, zdepw_b )

      !! compute correction if conservation needed
      IF ( ln_hsb ) THEN
         IF( iscpl_alloc() /= 0 )   CALL ctl_stop( 'STOP', 'rst_iscpl : unable to allocate rst_iscpl arrays' )
         CALL iscpl_cons(ztmask_b, zsmask_b, ze3t_b, htsc_iscpl, hdiv_iscpl, rdt_iscpl)
      END IF
         
      !! print mesh/mask
      IF( nmsh /= 0 .AND. ln_iscpl )   CALL dom_wri      ! Create a domain file

      IF ( ln_hsb ) THEN
         cfile='correction'
         cfile = TRIM( cfile )
         CALL iom_open  ( cfile, inum0, ldwrt = .TRUE., kiolib = jprstlib )
         CALL iom_rstput( 0, 0, inum0, 'vol_cor', hdiv_iscpl(:,:,:) )
         CALL iom_rstput( 0, 0, inum0, 'tem_cor', htsc_iscpl(:,:,:,jp_tem) )
         CALL iom_rstput( 0, 0, inum0, 'sal_cor', htsc_iscpl(:,:,:,jp_sal) )
         CALL iom_rstput( 0, 0, inum0, 'e3t_diff', (e3t_0(:,:,:)*tmask(:,:,:) - ze3t_b(:,:,:)*ztmask_b(:,:,:))*tmask(:,:,:) )
         CALL iom_close ( inum0 )
      END IF

      CALL wrk_dealloc(jpi,jpj,jpk, ztmask_b,zumask_b,zvmask_b )  
      CALL wrk_dealloc(jpi,jpj,jpk, ze3t_b  ,ze3u_b  ,ze3v_b   )  
      CALL wrk_dealloc(jpi,jpj,jpk, zdepw_b                    )
      CALL wrk_dealloc(jpi,jpj,     zsmask_b                   )
 
      neuler = 0              ! Euler restart (neuler=0)

      tsb    (:,:,:,:) = tsn    (:,:,:,:)  ! all before fields set to now values
      ub     (:,:,:)   = un     (:,:,:)
      vb     (:,:,:)   = vn     (:,:,:)
      rotb   (:,:,:)   = rotn   (:,:,:)
      hdivb  (:,:,:)   = hdivn  (:,:,:)
      fse3t_b(:,:,:)   = fse3t_n(:,:,:)
      sshb   (:,:)     = sshn   (:,:)
      !
   END SUBROUTINE iscpl_stp
   
   SUBROUTINE iscpl_rst_interpol (ptmask_b, pumask_b, pvmask_b, psmask_b, pe3t_b, pe3u_b, pe3v_b, pdepw_b)
      !!---------------------------------------------------------------------- 
      !!                   ***  ROUTINE iscpl_rst_interpol  ***
      !! 
      !! ** Purpose :   compute new tn, sn, un, vn and sshn in case of evolving geometry of ice shelves 
      !!                compute 2d fields of heat, salt and volume correction
      !! 
      !! ** Method  :   tn, sn : extrapolation from neigbourg cells
      !!                un, vn : fill with 0 velocity and keep barotropic transport by modifing surface velocity or adjacent velocity
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(:,:,:  ), INTENT(in ) :: ptmask_b, pumask_b, pvmask_b    !! mask before
      REAL(wp), DIMENSION(:,:,:  ), INTENT(in ) :: pe3t_b  , pe3u_b  , pe3v_b      !! scale factor before
      REAL(wp), DIMENSION(:,:,:  ), INTENT(in ) :: pdepw_b
      REAL(wp), DIMENSION(:,:    ), INTENT(in ) :: psmask_b                        !! mask before
      !!
      INTEGER :: ji, jj, jk, iz          !! loop index
      INTEGER :: jip1, jim1, jjp1, jjm1, jkp1, jkm1
      !!
      REAL(wp):: summsk, zsum, zsum1, zarea, zsumn, zsumb
      REAL(wp):: zdz, zdzm1, zdzp1
      !!
      REAL(wp), DIMENSION(:,:    ), POINTER :: zdmask , zdsmask, zvcorr, zucorr, zde3t
      REAL(wp), DIMENSION(:,:    ), POINTER :: zbub   , zbvb   , zbun  , zbvn
      REAL(wp), DIMENSION(:,:    ), POINTER :: zssh0  , zssh1, hu1, hv1
      REAL(wp), DIMENSION(:,:    ), POINTER :: zsmask0, zsmask1
      REAL(wp), DIMENSION(:,:,:  ), POINTER :: ztmask0, ztmask1, ztrp
      REAL(wp), DIMENSION(:,:,:  ), POINTER :: zwmaskn, zwmaskb, ztmp3d
      REAL(wp), DIMENSION(:,:,:,:), POINTER :: zts0
      !!
      CALL wrk_alloc(jpi,jpj,jpk,2, zts0                                   )
      CALL wrk_alloc(jpi,jpj,jpk,   ztmask0, ztmask1 , ztrp, ztmp3d         ) 
      CALL wrk_alloc(jpi,jpj,jpk,   zwmaskn, zwmaskb                       ) 
      CALL wrk_alloc(jpi,jpj,       zsmask0, zsmask1                       ) 
      CALL wrk_alloc(jpi,jpj,       zdmask , zdsmask, zvcorr, zucorr, zde3t) 
      CALL wrk_alloc(jpi,jpj,       zbub   , zbvb    , zbun , zbvn         ) 
      CALL wrk_alloc(jpi,jpj,       zssh0  , zssh1, hu1, hv1               ) 
      !! mask value to be sure
      tsn(:,:,:,jp_tem) = tsn(:,:,:,jp_tem) * ptmask_b(:,:,:)
      tsn(:,:,:,jp_sal) = tsn(:,:,:,jp_sal) * ptmask_b(:,:,:)
      
      ! compute wmask
      zwmaskn(:,:,1) = tmask   (:,:,1)
      zwmaskb(:,:,1) = ptmask_b(:,:,1)
      DO jk = 2,jpk
         zwmaskn(:,:,jk) =  tmask  (:,:,jk) *  tmask  (:,:,jk-1)
         zwmaskb(:,:,jk) = ptmask_b(:,:,jk) * ptmask_b(:,:,jk-1)
      END DO
           
      ! compute ssh     
      zsmask0(:,:) = psmask_b(:,:)
      zsmask1(:,:) = psmask_b(:,:)
      
      ! compute new ssh if we open a full water column (average of the closest neigbourgs)  
      sshb (:,:)=sshn(:,:)
      zssh0(:,:)=sshn(:,:)
      DO iz = 1,10    ! need to be tuned (configuration dependent)
         zdsmask(:,:) = ssmask(:,:)-zsmask0(:,:)
         DO ji = 2,jpi-1
            DO jj = 2,jpj-1
               jip1=ji+1; jim1=ji-1;
               jjp1=jj+1; jjm1=jj-1;
               summsk=(zsmask0(jip1,jj)+zsmask0(jim1,jj)+zsmask0(ji,jjp1)+zsmask0(ji,jjm1))
               IF (zdsmask(ji,jj)==1 .AND. summsk .NE. 0) THEN
                  PRINT *, 'add ssh to 1 cell',ji,jj,narea
                  sshn(ji,jj)=(zssh0(jip1,jj)*zsmask0(jip1,jj)     &
                  &           +zssh0(jim1,jj)*zsmask0(jim1,jj)     &
                  &           +zssh0(ji,jjp1)*zsmask0(ji,jjp1)     &
                  &           +zssh0(ji,jjm1)*zsmask0(ji,jjm1))/summsk
                  zsmask1(ji,jj)=1
               END IF
            END DO
         END DO
         CALL lbc_lnk(sshn,'T',1.)
         CALL lbc_lnk(zsmask1,'T',1.)
         zssh0   = sshn
         zsmask0 = zsmask1
      END DO
      sshn(:,:) = sshn(:,:) * ssmask(:,:)

!=============================================================================
! WARNING we cannot used glob_sum for before time variable (because glob_sum use tmask_i). Need to mask the halo and glob_sum_full
      ztmp3d(:,:,:) = 0.0
      ztmp3d(2:jpi-1,2:jpj-1,:) = fse3t_n(2:jpi-1,2:jpj-1,:) * tmask(2:jpi-1,2:jpj-1,:) &
      &                         - pe3t_b(2:jpi-1,2:jpj-1,:) * ptmask_b(2:jpi-1,2:jpj-1,:)
      zsum = glob_sum_full(ztmp3d) 
      IF (lwp) PRINT *, 'total volume correction 00 = ',zsum
      IF ( lk_vvl ) THEN
! compute fse3t_n
         DO jk = 1,jpk
            fse3t_n(:,:,jk) = e3t_0(:,:,jk) * ( 1 + sshn(:,:) / ( ht_0(:,:) + 1. - ssmask(:,:) ) * tmask(:,:,jk) )
         END DO
         ztmp3d(:,:,:) = 0.0
         ztmp3d(2:jpi-1,2:jpj-1,:) = fse3t_n(2:jpi-1,2:jpj-1,:) * tmask(2:jpi-1,2:jpj-1,:) &
         &                         - pe3t_b(2:jpi-1,2:jpj-1,:) * ptmask_b(2:jpi-1,2:jpj-1,:)
         zsum = glob_sum_full(ztmp3d) 
         IF (lwp) PRINT *, 'total volume correction 01 = ',zsum

! compute fse3u/v ... (call interpolation vvl)
      ! Reconstruction of all vertical scale factors at now and before time steps
      ! =============================================================================
      ! Horizontal scale factor interpolations
      ! --------------------------------------
         CALL dom_vvl_interpol( fse3t_n(:,:,:), fse3u_n(:,:,:), 'U' )
         CALL dom_vvl_interpol( fse3t_n(:,:,:), fse3v_n(:,:,:), 'V' )
         CALL dom_vvl_interpol( fse3u_n(:,:,:), fse3f_n(:,:,:), 'F' )

      ! Vertical scale factor interpolations
      ! ------------------------------------
         CALL dom_vvl_interpol( fse3t_n(:,:,:), fse3w_n (:,:,:), 'W'  )
         CALL dom_vvl_interpol( fse3u_n(:,:,:), fse3uw_n(:,:,:), 'UW' )
         CALL dom_vvl_interpol( fse3v_n(:,:,:), fse3vw_n(:,:,:), 'VW' )

      ! t- and w- points depth
      ! ----------------------
         fsdept_n(:,:,1) = 0.5_wp * fse3w_n(:,:,1)
         fsdepw_n(:,:,1) = 0.0_wp
         fsde3w_n(:,:,1) = fsdept_n(:,:,1) - sshn(:,:)
         DO jj = 1,jpj
            DO ji = 1,jpi
               DO jk = 2,mikt(ji,jj)-1
                  fsdept_n(ji,jj,jk) = gdept_0(ji,jj,jk)
                  fsdepw_n(ji,jj,jk) = gdepw_0(ji,jj,jk)
                  fsde3w_n(ji,jj,jk) = gdept_0(ji,jj,jk) - sshn(ji,jj)
               END DO
               IF (mikt(ji,jj) .GT. 1) THEN
                  jk = mikt(ji,jj)
                  fsdept_n(ji,jj,jk) = gdepw_0(ji,jj,jk) + 0.5_wp * fse3w_n(ji,jj,jk)
                  fsdepw_n(ji,jj,jk) = gdepw_0(ji,jj,jk)
                  fsde3w_n(ji,jj,jk) = fsdept_n(ji,jj,jk  ) - sshn   (ji,jj)
               END IF
               DO jk = mikt(ji,jj)+1, jpk
                  fsdept_n(ji,jj,jk) = fsdept_n(ji,jj,jk-1) + fse3w_n(ji,jj,jk)
                  fsdepw_n(ji,jj,jk) = fsdepw_n(ji,jj,jk-1) + fse3t_n(ji,jj,jk-1)
                  fsde3w_n(ji,jj,jk) = fsdept_n(ji,jj,jk  ) - sshn   (ji,jj)
               END DO
            END DO
         END DO

         hu(:,:) = 0._wp                          ! Ocean depth at U-points
         hv(:,:) = 0._wp                          ! Ocean depth at V-points
         ht(:,:) = 0._wp                          ! Ocean depth at T-points
         DO jk = 1, jpkm1
            hu(:,:) = hu(:,:) + fse3u_n(:,:,jk) * umask(:,:,jk)
            hv(:,:) = hv(:,:) + fse3v_n(:,:,jk) * vmask(:,:,jk)
            ht(:,:) = ht(:,:) + fse3t_n(:,:,jk) * tmask(:,:,jk)
         END DO
         !                                        ! Inverse of the local depth
         hur(:,:) = 1._wp / ( hu(:,:) + 1._wp - ssumask(:,:) ) * ssumask(:,:)
         hvr(:,:) = 1._wp / ( hv(:,:) + 1._wp - ssvmask(:,:) ) * ssvmask(:,:)

      END IF
      ztmp3d(:,:,:) = 0.0
      ztmp3d(2:jpi-1,2:jpj-1,:) = fse3t_n(2:jpi-1,2:jpj-1,:) * tmask(2:jpi-1,2:jpj-1,:) &
      &                         - pe3t_b(2:jpi-1,2:jpj-1,:) * ptmask_b(2:jpi-1,2:jpj-1,:)
      zsum = glob_sum_full(ztmp3d) 
      IF (lwp) PRINT *, 'total volume correction 02 = ',zsum

!=============================================================================

! compute velocity
! compute velocity in order to conserve barotropic velocity (modification by poderation of the scale factor).
      ub(:,:,:)=un(:,:,:)
      vb(:,:,:)=vn(:,:,:)
      DO jk = 1,jpk
         DO ji = 1,jpi
            DO jj = 1,jpj
               un(ji,jj,jk) = ub(ji,jj,jk)*pe3u_b(ji,jj,jk)*pumask_b(ji,jj,jk)/fse3u_n(ji,jj,jk)*umask(ji,jj,jk);
               vn(ji,jj,jk) = vb(ji,jj,jk)*pe3v_b(ji,jj,jk)*pvmask_b(ji,jj,jk)/fse3v_n(ji,jj,jk)*vmask(ji,jj,jk);
            END DO
         END DO
      END DO
! compute new velocity if we close a cell (check barotropic velocity and change velocity over the water column)
! compute barotropic velocity now and after 
      ztrp(:,:,:) = ub(:,:,:)*pe3u_b(:,:,:); 
      zbub(:,:)   = SUM(ztrp,DIM=3)
      ztrp(:,:,:) = vb(:,:,:)*pe3v_b(:,:,:); 
      zbvb(:,:)   = SUM(ztrp,DIM=3)
      ztrp(:,:,:) = un(:,:,:)*fse3u_n(:,:,:); 
      zbun(:,:)   = SUM(ztrp,DIM=3)
      ztrp(:,:,:) = vn(:,:,:)*fse3v_n(:,:,:); 
      zbvn(:,:)   = SUM(ztrp,DIM=3)
! Already know ????????
      hu1=0.0_wp ;
      hv1=0.0_wp ;
      DO jk  = 1,jpk
        hu1(:,:) = hu1(:,:) + fse3u_n(:,:,jk) * umask(:,:,jk)
        hv1(:,:) = hv1(:,:) + fse3v_n(:,:,jk) * vmask(:,:,jk)
      END DO
      zucorr = 0._wp
      zvcorr = 0._wp
      DO ji = 1,jpi
         DO jj = 1,jpj
            IF (zbun(ji,jj) .NE. zbub(ji,jj) .AND. hu1(ji,jj) .NE. 0._wp ) THEN
               zucorr(ji,jj) = (zbun(ji,jj) - zbub(ji,jj))/hu1(ji,jj)
            END IF
            IF (zbvn(ji,jj) .NE. zbvb(ji,jj) .AND. hv1(ji,jj) .NE. 0._wp ) THEN
               zvcorr(ji,jj) = (zbvn(ji,jj) - zbvb(ji,jj))/hv1(ji,jj)
            END IF
         END DO
      END DO 
      DO jk  = 1,jpk
         un(:,:,jk)=(un(:,:,jk) - zucorr(:,:))*umask(:,:,jk)
         vn(:,:,jk)=(vn(:,:,jk) - zvcorr(:,:))*vmask(:,:,jk)
      END DO

!=============================================================================
      ! compute temp and salt
      ! compute new tn and sn if we open a new cell
      tsb (:,:,:,:) = tsn(:,:,:,:)
      zts0(:,:,:,:) = tsn(:,:,:,:)
      ztmask1(:,:,:) = ptmask_b(:,:,:)
      ztmask0(:,:,:) = ptmask_b(:,:,:)
      DO iz = 1,10
          DO jk = 1,jpk-1
              zdmask=tmask(:,:,jk)-ztmask0(:,:,jk);
              DO ji = 2,jpi-1
                  DO jj = 2,jpj-1
                      jip1=ji+1; jim1=ji-1;
                      jjp1=jj+1; jjm1=jj-1;
                      summsk=(ztmask0(jip1,jj,jk)+ztmask0(jim1,jj,jk)+ztmask0(ji,jjp1,jk)+ztmask0(ji,jjm1,jk))
                      !! horizontal extrapolation
                      IF (zdmask(ji,jj)==1 .AND. summsk .NE. 0) THEN
                         tsn(ji,jj,jk,1)=( zts0(jip1,jj,jk,1)*ztmask0(jip1,jj,jk)     &
                         &                +zts0(jim1,jj,jk,1)*ztmask0(jim1,jj,jk)     &
                         &                +zts0(ji,jjp1,jk,1)*ztmask0(ji,jjp1,jk)     &
                         &                +zts0(ji,jjm1,jk,1)*ztmask0(ji,jjm1,jk))/summsk
                         tsn(ji,jj,jk,2)=( zts0(jip1,jj,jk,2)*ztmask0(jip1,jj,jk)     &
                         &                +zts0(jim1,jj,jk,2)*ztmask0(jim1,jj,jk)     &
                         &                +zts0(ji,jjp1,jk,2)*ztmask0(ji,jjp1,jk)     &
                         &                +zts0(ji,jjm1,jk,2)*ztmask0(ji,jjm1,jk))/summsk
                         ztmask1(ji,jj,jk)=1
                      END IF
                      !! vertical extrapolation if horizontal extra failed
                      IF (zdmask(ji,jj)==1 .AND. summsk==0) THEN
                         jkm1=max(1,jk-1) ; jkp1=min(jpk,jk+1)
                         summsk=(ztmask0(ji,jj,jkm1)+ztmask0(ji,jj,jkp1))
                         IF (zdmask(ji,jj)==1 .AND. summsk .NE. 0 ) THEN
                            tsn(ji,jj,jk,1)=( zts0(ji,jj,jkp1,1)*ztmask0(ji,jj,jkp1)     &
                            &                +zts0(ji,jj,jkm1,1)*ztmask0(ji,jj,jkm1))/summsk
                            tsn(ji,jj,jk,2)=( zts0(ji,jj,jkp1,2)*ztmask0(ji,jj,jkp1)     &
                            &                +zts0(ji,jj,jkm1,2)*ztmask0(ji,jj,jkm1))/summsk
                            ztmask1(ji,jj,jk)=1
                         END IF
                      END IF
                      !! horizontal corner extrapolation if horizontal and vertical failed
                      IF (zdmask(ji,jj)==1 .AND. summsk==0) THEN
                         jip1=ji+1; jim1=ji-1;
                         jjp1=jj+1; jjm1=jj-1;
                         summsk=(ztmask0(jip1,jjp1,jk)+ztmask0(jim1,jjm1,jk)+ztmask0(jim1,jjp1,jk)+ztmask0(jip1,jjm1,jk))
                         IF (zdmask(ji,jj)==1 .AND. summsk .NE. 0) THEN
                            tsn(ji,jj,jk,1)=( zts0(jip1,jjp1,jk,1)*ztmask0(jip1,jjp1,jk)     &
                            &                +zts0(jim1,jjm1,jk,1)*ztmask0(jim1,jjm1,jk)     &
                            &                +zts0(jim1,jjp1,jk,1)*ztmask0(jim1,jjp1,jk)     &
                            &                +zts0(jip1,jjm1,jk,1)*ztmask0(jip1,jjm1,jk))/summsk
                            tsn(ji,jj,jk,2)=( zts0(jip1,jjp1,jk,2)*ztmask0(jip1,jjp1,jk)     &
                            &                +zts0(jim1,jjm1,jk,2)*ztmask0(jim1,jjm1,jk)     &
                            &                +zts0(jim1,jjp1,jk,2)*ztmask0(jim1,jjp1,jk)     &
                            &                +zts0(jip1,jjm1,jk,2)*ztmask0(jip1,jjm1,jk))/summsk
                            ztmask1(ji,jj,jk)=1
                         END IF
                      END IF
                  END DO
              END DO
          END DO
          CALL lbc_lnk(tsn(:,:,:,1),'T',1.)
          CALL lbc_lnk(tsn(:,:,:,2),'T',1.)
          CALL lbc_lnk(ztmask1,'T',1.)
          zts0(:,:,:,:) = tsn(:,:,:,:)
          ztmask0 = ztmask1
      END DO
      tsn(:,:,:,jp_tem) = tsn(:,:,:,jp_tem) * tmask(:,:,:)
      tsn(:,:,:,jp_sal) = tsn(:,:,:,jp_sal) * tmask(:,:,:)

      ! CHECK heat
      zsumn = glob_sum( tsn(:,:,:,jp_tem) * fse3t_n(:,:,:) * zwmaskb(:,:,:) * zwmaskn(:,:,:))
      zsumb = glob_sum( tsb(:,:,:,jp_tem) *  pe3t_b(:,:,:) * zwmaskb(:,:,:) * zwmaskn(:,:,:))
      IF (lwp) PRINT *, 'CHECK tsn = ',zsumn, zsumb

      ! compute new T/S (interpolation) because of vvl
      IF ( lk_vvl .AND. .FALSE. ) THEN
      !IF ( lk_vvl ) THEN
         DO jk = 2,jpk-1
            DO jj = 1,jpj
               DO ji = 1,jpi
                  IF (zwmaskn(ji,jj,jk) * zwmaskb(ji,jj,jk) == 1.0_wp ) THEN
                     !compute weight
                     zdzp1 = MAX(0._wp,fsdepw_n(ji,jj,jk+1) - pdepw_b(ji,jj,jk+1))
                     zdz   =           fsdepw_n(ji,jj,jk+1) - pdepw_b(ji,jj,jk  ) 
                     zdzm1 = MAX(0._wp,pdepw_b(ji,jj,jk  ) - fsdepw_n(ji,jj,jk  ))
                     IF (zdz .LT. 0.0_wp) THEN 
                        PRINT *, 'ERROR dz n ', ji,jj,jk,zdz,fsdepw_n(ji,jj,jk+1),fsdepw_n(ji,jj,jk),fsdepw_n(ji,jj,jk-1)
                        PRINT *, 'ERROR dz n             = ',fse3t_n (ji,jj,jk+1),fse3t_n (ji,jj,jk),fse3t_n (ji,jj,jk-1), sshn(ji,jj)
                        PRINT *, 'ERROR dz b             = ',pdepw_b(ji,jj,jk+1),pdepw_b(ji,jj,jk),pdepw_b(ji,jj,jk-1)
                        PRINT *, 'ERROR dz b             = ',fse3t_b (ji,jj,jk+1),fse3t_b (ji,jj,jk),fse3t_b (ji,jj,jk-1), sshb(ji,jj)
                        PRINT *, 'ERROR dz 0             = ',  e3t_0 (ji,jj,jk+1),  e3t_0 (ji,jj,jk),  e3t_0 (ji,jj,jk-1)
                        PRINT *, 'ERROR dz n             = ',  tmask (ji,jj,jk+1),  tmask (ji,jj,jk),  tmask (ji,jj,jk-1)
                        PRINT *, 'ERROR dz n             = ', zwmaskn(ji,jj,jk+1), zwmaskn(ji,jj,jk), zwmaskn(ji,jj,jk-1)
                        PRINT *, 'ERROR dz b             = ', ptmask_b(ji,jj,jk+1), ptmask_b(ji,jj,jk), ptmask_b(ji,jj,jk-1)
                        PRINT *, 'ERROR dz b             = ', zwmaskb(ji,jj,jk+1), zwmaskb(ji,jj,jk), zwmaskb(ji,jj,jk-1)
                        PRINT *, 'ERROR dz b             = ', gdepw_0(ji,jj,jk+1), gdepw_0(ji,jj,jk), gdepw_0(ji,jj,jk-1)
                        STOP
                     END IF
                     tsn(ji,jj,jk,jp_tem) = ( zdzp1*tsb(ji,jj,jk+1,jp_tem)      &
                        &                   + zdz  *tsb(ji,jj,jk  ,jp_tem)      &
                        &                   + zdzm1*tsb(ji,jj,jk-1,jp_tem)      )/fse3t_n(ji,jj,jk)
                     tsn(ji,jj,jk,jp_sal) = ( zdzp1*tsb(ji,jj,jk+1,jp_sal)      &
                        &                   + zdz  *tsb(ji,jj,jk  ,jp_sal)      &
                        &                   + zdzm1*tsb(ji,jj,jk-1,jp_sal)      )/fse3t_n(ji,jj,jk)
                  END IF
               END DO
            END DO
         END DO               
      END IF

      ! CHECK heat
      ztmp3d(:,:,:) = 0.0
      ztmp3d(2:jpi-1,2:jpj-1,:) = fse3t_n(2:jpi-1,2:jpj-1,:) * tmask(2:jpi-1,2:jpj-1,:) &
      &                         - pe3t_b(2:jpi-1,2:jpj-1,:) * ptmask_b(2:jpi-1,2:jpj-1,:)
      zsum = glob_sum_full(ztmp3d) 
      IF (lwp) PRINT *, 'total volume correction 03 = ',zsum

      WHERE (tmask(:,:,:) == 1.0 .AND. tsn(:,:,:,2) == 0._wp) 
         tsn(:,:,:,2)=  -99._wp
         tmask(:,:,:) = 0.0
         umask(:,:,:) = 0.0
         vmask(:,:,:) = 0.0
      END WHERE

      WHERE (SUM(tmask,dim=3) == 0)
         mbkt(:,:)=1 ; mbku(:,:)=1 ; mbkv(:,:)=1
         mikt(:,:)=1 ; miku(:,:)=1 ; mikv(:,:)=1
      END WHERE

      ! compute new tn and sn if we close cell 
      ! nothing to do

      !! next step is an euler time step
      neuler = 0 
      !! set _b and _n variables equal
      tsb (:,:,:,:)=tsn (:,:,:,:)
      ub  (:,:,:  )=un  (:,:,:  )
      vb  (:,:,:  )=vn  (:,:,:  )
      sshb(:,:    )=sshn(:,:)
      !! set _b and _n vertical scale factor equal
      fse3t_b (:,:,:)=fse3t_n (:,:,:)
      fse3u_b (:,:,:)=fse3u_n (:,:,:)
      fse3v_b (:,:,:)=fse3v_n (:,:,:)
      IF ( lk_vvl ) THEN
         fse3uw_b(:,:,:) = fse3uw_n(:,:,:)
         fse3vw_b(:,:,:) = fse3vw_n(:,:,:)
         fsdept_b(:,:,:) = fsdept_n(:,:,:)
         fsdepw_b(:,:,:) = fsdepw_n(:,:,:)
         hu_b(:,:) = hu(:,:)
         hv_b(:,:) = hv(:,:)
         hur_b(:,:) = hur(:,:)
         hvr_b(:,:) = hvr(:,:)
      END IF
      !! 
      CALL wrk_dealloc(jpi,jpj,jpk,2, zts0                                   )
      CALL wrk_dealloc(jpi,jpj,jpk,   ztmask0, ztmask1 , ztrp                ) 
      CALL wrk_dealloc(jpi,jpj,jpk,   zwmaskn, zwmaskb , ztmp3d              ) 
      CALL wrk_dealloc(jpi,jpj,       zsmask0, zsmask1                       ) 
      CALL wrk_dealloc(jpi,jpj,       zdmask , zdsmask, zvcorr, zucorr, zde3t) 
      CALL wrk_dealloc(jpi,jpj,       zbub   , zbvb    , zbun  , zbvn        ) 
      CALL wrk_dealloc(jpi,jpj,       zssh0  , zssh1  , hu1 , hv1            ) 

   END SUBROUTINE iscpl_rst_interpol

END MODULE iscplrst
